"""Knowledge representation module — Universal Knowledge Representation (UKR)."""

from model_osmosis.knowledge.representation import (
    UniversalKnowledge,
    StructuralKnowledge,
    FunctionalKnowledge,
    MetaKnowledge,
)
from model_osmosis.knowledge.storage import save_knowledge, load_knowledge

__all__ = [
    "UniversalKnowledge",
    "StructuralKnowledge",
    "FunctionalKnowledge",
    "MetaKnowledge",
    "save_knowledge",
    "load_knowledge",
]
